@@ -220,6 +220,23 @@ static void con_ic_connection_dispose(GObject *obj)
 	G_OBJECT_CLASS(parent_class)->dispose(obj);
 }
 
+static void con_ic_connection_clear_proxy_settings(ConIcConnection *connection)
+{
+	connection->proxy_mode = CON_IC_PROXY_MODE_NONE;
+	g_free(connection->http_host); connection->http_host = NULL;
+	g_free(connection->ftp_host); connection->ftp_host = NULL;
+	g_free(connection->https_host); connection->https_host = NULL;
+	g_free(connection->socks_host); connection->socks_host = NULL;
+	g_free(connection->rtsp_host); connection->rtsp_host = NULL;
+	g_free(connection->autoconfig_url); connection->autoconfig_url = NULL;
+
+	connection->http_port = 0;
+	connection->ftp_port = 0;
+	connection->https_port = 0;
+	connection->socks_port = 0;
+	connection->rtsp_port = 0;
+}
+
 static void con_ic_connection_finalize(GObject *obj)
 {
 	ConIcConnection *connection = CON_IC_CONNECTION(obj);
@@ -230,14 +247,7 @@ static void con_ic_connection_finalize(GObject *obj)
 		        (GFunc)dbus_pending_call_unref, NULL);
 	g_slist_free(connection->pending_calls);
 	
-	g_free(connection->iap_id);
-	g_free(connection->bearer);
-	g_free(connection->http_host);
-	g_free(connection->ftp_host);
-	g_free(connection->https_host);
-	g_free(connection->socks_host);
-	g_free(connection->rtsp_host);
-	g_free(connection->autoconfig_url);
+	con_ic_connection_clear_proxy_settings(connection);
 
 	objects = g_slist_remove(objects, connection);
 	if (objects == NULL) {
@@ -320,7 +330,8 @@ static gboolean con_ic_connection_enable_automatic_events(ConIcConnection *conne
 	}
 
 	if (!dbus_connection_send_with_reply(dbus_conn, msg,
-					     &pending, 1000)) {
+					     &pending, 1000) ||
+	    pending == NULL) {
 		ret = FALSE;
 		goto exit;
 	}
@@ -429,6 +440,8 @@ static DBusHandlerResult con_ic_connection_handle_icd_message(DBusConnection *c,
 		status = CON_IC_STATUS_DISCONNECTED;
 	} else if (strcmp(state, "CONNECTED") == 0) {
 		status = CON_IC_STATUS_CONNECTED;
+	} else if (strcmp(state, "NETWORKUP") == 0) {
+		status = CON_IC_STATUS_NETWORK_UP;
 	} else {
 		goto exit;
 	}
@@ -444,16 +457,39 @@ static DBusHandlerResult con_ic_connection_handle_icd_message(DBusConnection *c,
 	for (e = copy; e != NULL; e = g_slist_next(e)) {
 		ConIcConnection *connection = e->data;
 
+		/* Clear cached proxy settings, if connections are changed */
+		if (status == CON_IC_STATUS_CONNECTED ||
+		    status == CON_IC_STATUS_DISCONNECTED)
+			con_ic_connection_clear_proxy_settings(connection);
+
 		/* if automatic events are enabled, application has to track
 		   the connections itself */
-		if (connection->automatic_events)
-		  con_ic_connection_send_event(connection,
-					       iap_id,
-					       bearer,
-					       status);
+		if (connection->automatic_events) {
+			if (status == CON_IC_STATUS_CONNECTED) {
+				/* If IAP was connected, start tracking the 
+				   new IAP */
+				ConIcIap *iap;
+				connection->internal_status = CONNECTED;
+				g_free(connection->iap_id);
+				connection->iap_id = g_strdup(iap_id);
+
+				g_free(connection->bearer);
+				iap = con_ic_iap_new(connection->iap_id);
+				if (iap) {
+				  connection->bearer =
+				    g_strdup(con_ic_iap_get_bearer_type(iap));
+				  g_object_unref(iap);
+				} else
+				  connection->bearer = NULL;
+			}
+			con_ic_connection_send_event(connection,
+						     iap_id,
+						     bearer,
+						     status);
+		}
 
 		switch (status) {
-		case CON_IC_STATUS_CONNECTED:		
+		case CON_IC_STATUS_CONNECTED:
 		  break;
 
 		case CON_IC_STATUS_DISCONNECTED:		
@@ -462,13 +498,18 @@ static DBusHandlerResult con_ic_connection_handle_icd_message(DBusConnection *c,
 		      connection->iap_id != NULL &&
 		      strcmp(iap_id, connection->iap_id) == 0) {
 
-		    if (!connection->automatic_events)
-		      con_ic_connection_send_event(connection,
-						   connection->iap_id,
-						   connection->bearer,
-						   status);
+			if (!connection->automatic_events)
+				con_ic_connection_send_event(
+							connection,
+							connection->iap_id,
+							connection->bearer,
+							status);
 
-		    connection->internal_status = DISCONNECTED;
+			connection->internal_status = DISCONNECTED;
+			g_free(connection->iap_id);
+			connection->iap_id = NULL;
+			g_free(connection->bearer);
+			connection->bearer = NULL;
 		  }
 		  break;
 
@@ -553,15 +594,31 @@ static void con_ic_connection_connected_cb(DBusPendingCall *pending,
 	if (connection->internal_status != CONNECTING)
 		goto exit;
 
+#if 0
+	/* Broken applications are broken by not sending an extra
+	   connected signal, revert the fix. Now applications must
+	   track all connected IAPs in order to tell whether we are
+	   online or offline.
+	*/
+	/* if automatic events are enabled, we already get the
+	   connected signal via that, don't send two connected
+	   events */
+	if (connection->automatic_events)
+		goto exit;
+#endif
+	
 	connection->internal_status = CONNECTED;
 
 	g_free(connection->iap_id);
 	connection->iap_id = g_strdup(iap_id);
 
-	iap = con_ic_iap_new(connection->iap_id);
 	g_free(connection->bearer);
-	connection->bearer = g_strdup(con_ic_iap_get_bearer_type(iap));
-	g_object_unref(iap);
+	iap = con_ic_iap_new(connection->iap_id);
+	if (iap) {
+	  connection->bearer = g_strdup(con_ic_iap_get_bearer_type(iap));
+	  g_object_unref(iap);
+	} else
+	  connection->bearer = NULL;
 	
 	con_ic_connection_send_event(connection, connection->iap_id,
 				     connection->bearer,
@@ -778,7 +835,8 @@ gboolean con_ic_connection_connect_by_id(ConIcConnection *connection,
 	}
 
 	if (!dbus_connection_send_with_reply(dbus_conn, msg,
-					     &pending, 3*60*1000)) {
+					     &pending, 3*60*1000) ||
+	    pending == NULL) {
 		ret = FALSE;
 		goto exit;
 	}
@@ -868,7 +926,8 @@ gboolean con_ic_connection_disconnect_by_id(ConIcConnection *connection,
 	}
 
 	if (!dbus_connection_send_with_reply(dbus_conn, msg,
-					     &pending, 30*1000)) {
+					     &pending, 30*1000) ||
+	    pending == NULL) {
 		ret = FALSE;
 		goto exit;
 	}
@@ -920,7 +979,8 @@ gboolean con_ic_connection_statistics(ConIcConnection *connection,
 	}
 
 	if (!dbus_connection_send_with_reply(dbus_conn, msg,
-					     &pending, 1000)) {
+					     &pending, 1000) ||
+	    pending == NULL) {
 		ret = FALSE;
 		goto exit;
 	}
@@ -945,18 +1005,33 @@ exit:
 }
 
 
+static gchar *con_ic_connection_get_gconf_path(ConIcConnection *connection,
+					       const gchar *key)
+{
+	gchar *escaped = NULL;
+	gchar *path = NULL;
+
+	g_return_val_if_fail(connection != NULL, NULL);
+	if (connection->iap_id == NULL) return NULL;
+
+	escaped = gconf_escape_key (connection->iap_id, -1);
+	path = g_strconcat (ICD_GCONF_PATH, "/", escaped, "/", key, NULL );
+	g_free (escaped);
+	return path;
+}
+
+
 ConIcProxyMode con_ic_connection_get_proxy_mode(ConIcConnection *connection)
 {
 	char *s = NULL;
-	GConfClient *gconf = gconf_client_get_default();
-	char *escaped;
-	char *path;
+	GConfClient *gconf = NULL;
+	char *path = con_ic_connection_get_gconf_path(connection, "proxytype");
+
+	if (path == NULL) return CON_IC_PROXY_MODE_NONE;
 
 	/* FIXME: use dbus proxy signals as soon as icd sends them */
 
-	escaped = gconf_escape_key (connection->iap_id, -1);
-	path = g_strconcat (ICD_GCONF_PATH, "/", escaped, "/proxytype", NULL );
-	g_free (escaped);
+	gconf = gconf_client_get_default();
 	s = gconf_client_get_string(gconf, path, NULL);
 	g_free (path);
 
@@ -982,15 +1057,16 @@ exit:
 
 static const gchar *con_ic_connection_get_proxy_http_host(ConIcConnection *connection)
 {
-	GConfClient *gconf = gconf_client_get_default();
-	char *escaped = gconf_escape_key (connection->iap_id, -1);
-	char *path = g_strconcat (ICD_GCONF_PATH, "/", escaped,
-				  "/proxy_http", NULL);
+	GConfClient *gconf = NULL;
+	char *path = con_ic_connection_get_gconf_path(connection, 
+						      "proxy_http");
 
-	g_free (escaped);
+	if (path == NULL) return NULL;
 
 	/* FIXME: use dbus proxy signals as soon as icd sends them */
 
+	gconf = gconf_client_get_default();
+	g_free(connection->http_host);
 	connection->http_host = gconf_client_get_string(gconf,
 							path,
 							NULL);
@@ -1001,15 +1077,15 @@ static const gchar *con_ic_connection_get_proxy_http_host(ConIcConnection *conne
 
 static gint con_ic_connection_get_proxy_http_port(ConIcConnection *connection)
 {
-	GConfClient *gconf = gconf_client_get_default();
-	char *escaped = gconf_escape_key (connection->iap_id, -1);
-	char *path = g_strconcat (ICD_GCONF_PATH, "/", escaped,
-				  "/proxy_http_port", NULL);
+	GConfClient *gconf = NULL;
+	char *path = con_ic_connection_get_gconf_path(connection, 
+						      "proxy_http_port");
 
-	g_free (escaped);
+	if (path == NULL) return 0;
 
 	/* FIXME: use dbus proxy signals as soon as icd sends them */
 
+	gconf = gconf_client_get_default();
 	connection->http_port = gconf_client_get_int(gconf,
 						     path,
 						     NULL);
@@ -1020,15 +1096,16 @@ static gint con_ic_connection_get_proxy_http_port(ConIcConnection *connection)
 
 static const gchar *con_ic_connection_get_proxy_ftp_host(ConIcConnection *connection)
 {
-	GConfClient *gconf = gconf_client_get_default();
-	char *escaped = gconf_escape_key (connection->iap_id, -1);
-	char *path = g_strconcat (ICD_GCONF_PATH, "/", escaped,
-				  "/proxy_ftp", NULL);
+	GConfClient *gconf = NULL;
+	char *path = con_ic_connection_get_gconf_path(connection,
+						      "proxy_ftp");
 
-	g_free (escaped);
+	if (path == NULL) return NULL;
 
 	/* FIXME: use dbus proxy signals as soon as icd sends them */
 
+	gconf = gconf_client_get_default();
+	g_free(connection->ftp_host);
 	connection->ftp_host = gconf_client_get_string(gconf,
 						       path,
 						       NULL);
@@ -1039,15 +1116,15 @@ static const gchar *con_ic_connection_get_proxy_ftp_host(ConIcConnection *connec
 
 static gint con_ic_connection_get_proxy_ftp_port(ConIcConnection *connection)
 {
-	GConfClient *gconf = gconf_client_get_default();
-	char *escaped = gconf_escape_key (connection->iap_id, -1);
-	char *path = g_strconcat (ICD_GCONF_PATH, "/", escaped,
-				  "/proxy_ftp_port", NULL);
+	GConfClient *gconf = NULL;
+	char *path = con_ic_connection_get_gconf_path(connection,
+						      "proxy_ftp_port");
 
-	g_free (escaped);
+	if (path == NULL) return 0;
 
 	/* FIXME: use dbus proxy signals as soon as icd sends them */
 
+	gconf = gconf_client_get_default();
 	connection->ftp_port = gconf_client_get_int(gconf,
 						    path,
 						    NULL);
@@ -1058,15 +1135,16 @@ static gint con_ic_connection_get_proxy_ftp_port(ConIcConnection *connection)
 
 static const gchar *con_ic_connection_get_proxy_https_host(ConIcConnection *connection)
 {
-	GConfClient *gconf = gconf_client_get_default();
-	char *escaped = gconf_escape_key (connection->iap_id, -1);
-	char *path = g_strconcat (ICD_GCONF_PATH, "/", escaped,
-				  "/proxy_https", NULL);
+	GConfClient *gconf = NULL;
+	char *path = con_ic_connection_get_gconf_path(connection,
+						      "proxy_https");
 
-	g_free (escaped);
+	if (path == NULL) return NULL;
 
 	/* FIXME: use dbus proxy signals as soon as icd sends them */
 
+	gconf = gconf_client_get_default();
+	g_free(connection->https_host);
 	connection->https_host = gconf_client_get_string(gconf,
 							 path,
 							NULL);
@@ -1077,15 +1155,15 @@ static const gchar *con_ic_connection_get_proxy_https_host(ConIcConnection *conn
 
 static gint con_ic_connection_get_proxy_https_port(ConIcConnection *connection)
 {
-	GConfClient *gconf = gconf_client_get_default();
-	char *escaped = gconf_escape_key (connection->iap_id, -1);
-	char *path = g_strconcat (ICD_GCONF_PATH, "/", escaped,
-				  "/proxy_https_port", NULL);
+	GConfClient *gconf = NULL;
+	char *path = con_ic_connection_get_gconf_path (connection,
+						       "proxy_https_port");
 
-	g_free (escaped);
+	if (path == NULL) return 0;
 
 	/* FIXME: use dbus proxy signals as soon as icd sends them */
 
+	gconf = gconf_client_get_default();
 	connection->https_port = gconf_client_get_int(gconf,
 						      path,
 						      NULL);
@@ -1096,15 +1174,16 @@ static gint con_ic_connection_get_proxy_https_port(ConIcConnection *connection)
 
 static const gchar *con_ic_connection_get_proxy_socks_host(ConIcConnection *connection)
 {
-	GConfClient *gconf = gconf_client_get_default();
-	char *escaped = gconf_escape_key (connection->iap_id, -1);
-	char *path = g_strconcat (ICD_GCONF_PATH, "/", escaped,
-				  "/proxy_socks", NULL);
+	GConfClient *gconf = NULL;
+	char *path = con_ic_connection_get_gconf_path(connection,
+						      "proxy_socks");
 
-	g_free (escaped);
+	if (path == NULL) return NULL;
 
 	/* FIXME: use dbus proxy signals as soon as icd sends them */
 
+	gconf = gconf_client_get_default();
+	g_free(connection->socks_host);
 	connection->socks_host = gconf_client_get_string(gconf,
 							 path,
 							 NULL);
@@ -1115,15 +1194,15 @@ static const gchar *con_ic_connection_get_proxy_socks_host(ConIcConnection *conn
 
 static gint con_ic_connection_get_proxy_socks_port(ConIcConnection *connection)
 {
-	GConfClient *gconf = gconf_client_get_default();
-	char *escaped = gconf_escape_key (connection->iap_id, -1);
-	char *path = g_strconcat (ICD_GCONF_PATH, "/", escaped,
-				  "/proxy_socks_port", NULL);
+	GConfClient *gconf = NULL;
+	char *path = con_ic_connection_get_gconf_path (connection,
+						       "proxy_socks_port");
 
-	g_free (escaped);
+	if (path == NULL) return 0;
 
 	/* FIXME: use dbus proxy signals as soon as icd sends them */
 
+	gconf = gconf_client_get_default();
 	connection->socks_port = gconf_client_get_int(gconf,
 						      path,
 						      NULL);
@@ -1135,15 +1214,16 @@ static gint con_ic_connection_get_proxy_socks_port(ConIcConnection *connection)
 
 static const gchar *con_ic_connection_get_proxy_rtsp_host(ConIcConnection *connection)
 {
-	GConfClient *gconf = gconf_client_get_default();
-	char *escaped = gconf_escape_key (connection->iap_id, -1);
-	char *path = g_strconcat (ICD_GCONF_PATH, "/", escaped,
-				  "/proxy_rtsp", NULL);
+	GConfClient *gconf = NULL;
+	char *path = con_ic_connection_get_gconf_path(connection,
+						      "proxy_rtsp");
 
-	g_free (escaped);
+	if (path == NULL) return NULL;
 
 	/* FIXME: use dbus proxy signals as soon as icd sends them */
 
+	gconf = gconf_client_get_default();
+	g_free(connection->rtsp_host);
 	connection->rtsp_host = gconf_client_get_string(gconf,
 							path,
 							NULL);
@@ -1154,15 +1234,15 @@ static const gchar *con_ic_connection_get_proxy_rtsp_host(ConIcConnection *conne
 
 static gint con_ic_connection_get_proxy_rtsp_port(ConIcConnection *connection)
 {
-	GConfClient *gconf = gconf_client_get_default();
-	char *escaped = gconf_escape_key (connection->iap_id, -1);
-	char *path = g_strconcat (ICD_GCONF_PATH, "/", escaped,
-				  "/proxy_rtsp_port", NULL);
+	GConfClient *gconf = NULL;
+	char *path = con_ic_connection_get_gconf_path (connection,
+						       "proxy_rtsp_port");
 
-	g_free (escaped);
+	if (path == NULL) return 0;
 
 	/* FIXME: use dbus proxy signals as soon as icd sends them */
 
+	gconf = gconf_client_get_default();
 	connection->rtsp_port = gconf_client_get_int(gconf,
 						     path,
 						     NULL);
@@ -1174,22 +1254,22 @@ static gint con_ic_connection_get_proxy_rtsp_port(ConIcConnection *connection)
 
 const gchar *con_ic_connection_get_proxy_autoconfig_url(ConIcConnection *connection)
 {
-	GConfClient *gconf = gconf_client_get_default();
-	char *escaped = gconf_escape_key (connection->iap_id, -1);
-	char *path = g_strconcat (ICD_GCONF_PATH, "/", escaped,
-				  "/autoconf_url", NULL);
+	GConfClient *gconf = NULL;
+	char *path = con_ic_connection_get_gconf_path (connection,
+						       "autoconf_url");
 
-	g_free (escaped);
+	if (path == NULL) return NULL;
 
 	/* FIXME: use dbus proxy signals as soon as icd sends them */
 
+	gconf = gconf_client_get_default();
+	g_free(connection->http_host);
 	connection->http_host = gconf_client_get_string(gconf,
 							path,
 							NULL);
 	g_free (path);
 	g_object_unref(gconf);
 	return connection->http_host;
-	return NULL;
 }
 
 const gchar *con_ic_connection_get_proxy_host(ConIcConnection* connection,
@@ -1253,13 +1333,14 @@ gint con_ic_connection_get_proxy_port(ConIcConnection* connection,
 
 GSList *con_ic_connection_get_proxy_ignore_hosts(ConIcConnection *connection)
 {
-	GConfClient *gconf = gconf_client_get_default();
-	char *escaped = gconf_escape_key (connection->iap_id, -1);
-	char *path = g_strconcat (ICD_GCONF_PATH, "/", escaped,
-				  "/omit_proxy", NULL);
+	GConfClient *gconf = NULL;
+	char *path = con_ic_connection_get_gconf_path(connection, 
+						      "omit_proxy");
 	GSList *list;
 
-	g_free (escaped);
+	if (path == NULL) return NULL;
+
+	gconf = gconf_client_get_default();
 	list = gconf_client_get_list(gconf, path,
 				     GCONF_VALUE_STRING, NULL);
 	g_free (path);
@@ -1267,6 +1348,32 @@ GSList *con_ic_connection_get_proxy_ignore_hosts(ConIcConnection *connection)
 	return list;
 }
 
+
+/** Check wheter an IAP is temporary
+ * @param gconf_client GConfClient
+ * @param iap_key the full GConf IAP path name
+ * @param id the IAP id
+ * @return TRUE if temporary, FALSE otherwise
+ */
+static gboolean con_ic_connection_is_temporary (GConfClient *gconf_client,
+						const gchar *iap_key,
+						const gchar *id)
+{
+  gchar *key;
+  gboolean result = FALSE;
+  
+  if (id == NULL)
+    return FALSE;
+
+  key = g_strconcat (iap_key, "/temporary", NULL);
+  if (!strncmp (id, "[Easy", 5) ||
+      gconf_client_get_bool (gconf_client, key, NULL))
+    result = TRUE;
+
+  g_free (key);
+  return result;
+}
+
 GSList *con_ic_connection_get_all_iaps(ConIcConnection *connection)
 {
 	GConfClient *gconf = gconf_client_get_default();
@@ -1280,14 +1387,14 @@ GSList *con_ic_connection_get_all_iaps(ConIcConnection *connection)
 	/* GConf returns absolute directory names, so we just
 	 * return the relative directory names */
 	for (entry = list; entry; entry = entry->next) {
-		id = gconf_unescape_key(strrchr(entry->data, '/')+1, -1);
-		g_free(entry->data);
-		if (id != NULL &&
-		    strncmp (id, "[Easy", 5)) {
-		  if ((iap = con_ic_iap_new(id)) != NULL)
-		    newl = g_slist_prepend (newl, iap);
-		}
-		g_free (id);
+	  if (entry->data != NULL) {
+	    id = gconf_unescape_key(strrchr(entry->data, '/')+1, -1);
+	    if (!con_ic_connection_is_temporary (gconf, entry->data, id) &&
+		(iap = con_ic_iap_new(id)) != NULL)
+	      newl = g_slist_prepend (newl, iap);
+	    g_free (id);
+	    g_free(entry->data);
+	  }
 	}
 	g_slist_free (list);
 	newl = g_slist_reverse (newl);
