@@ -73,7 +73,9 @@ typedef enum {
 	/** The IAP was disconnected */
 	CON_IC_STATUS_DISCONNECTED,
 	/** The IAP is disconnecting */
-	CON_IC_STATUS_DISCONNECTING
+	CON_IC_STATUS_DISCONNECTING,
+	/** The IAP has a network address, but is not yet fully connected */
+	CON_IC_STATUS_NETWORK_UP
 } ConIcConnectionStatus;
 
 /**
