@@ -57,11 +57,14 @@ static void test_all_iaps()
 static void test_get_iap()
 {
 	ConIcIap *iap = con_ic_connection_get_iap(connection, "osso.net");
-	g_assert(CON_IC_IS_IAP(iap));
-	g_debug("get IAP osso.net: %s (%s) %s",
-		con_ic_iap_get_id(iap),
-		con_ic_iap_get_name(iap),
-		con_ic_iap_get_bearer_type(iap));
+	if (iap) {
+	  g_assert(CON_IC_IS_IAP(iap));
+	  g_debug("get IAP osso.net: %s (%s) %s",
+		  con_ic_iap_get_id(iap),
+		  con_ic_iap_get_name(iap),
+		  con_ic_iap_get_bearer_type(iap));
+	} else
+	  g_debug ("get IAP osso.net: osso.net does not exist");
 }
 
 static gboolean start(gpointer user_data)
